/*
 * PROJECT: matereal at http://mr.digitalmuseum.jp/
 * ----------------------------------------------------------------------------
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is matereal.
 *
 * The Initial Developer of the Original Code is Jun KATO.
 * Portions created by the Initial Developer are
 * Copyright (C) 2009 Jun KATO. All Rights Reserved.
 *
 * Contributor(s): Jun KATO
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */
/**
 *
 */
package sample.test.pen;

import java.awt.Graphics2D;
import java.awt.geom.GeneralPath;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import jp.digitalmuseum.utils.ScreenPosition;

public class Path implements Iterable<ScreenPosition> {
	private PointList points;
	private double length;

	public Path() {
		points = new PointList();
	}

	public synchronized void add(int x, int y) {
		points.add(x, y);
		calcLength();
	}

	public int size() {
		return points.size();
	}

	public double getLength() {
		return length;
	}

	public synchronized void resample(int unitLength) {
		final PointList newPoints = new PointList();
		final Iterator<ScreenPosition> it = points.iterator();
		ScreenPosition currentPosition = it.next(), nextPosition = null;
		while(it.hasNext()) {
			nextPosition = it.next();
			double distance;
			while ((distance = currentPosition.distance(nextPosition)) > unitLength) {
				newPoints.add(currentPosition);
				currentPosition = new ScreenPosition(
						(int) ((nextPosition.getX()*unitLength +
								currentPosition.getX()*(distance-unitLength))/distance),
						(int) ((nextPosition.getY()*unitLength +
								currentPosition.getY()*(distance-unitLength))/distance));
			}
		}
		if (nextPosition != null) {
			newPoints.add(nextPosition);
			points = newPoints;
		}
	}

	public synchronized void paint(Graphics2D g) {
		points.paint(g);
	}

	private void calcLength() {
		length = 0;
		ScreenPosition p_ = points.get(0);
		for (ScreenPosition p : points) {
			length += p_.distance(p);
			p_ = p;
		}
	}

	public Iterator<ScreenPosition> iterator() {
		return points.iterator();
	}

	private class PointList implements Iterable<ScreenPosition> {
		private List<ScreenPosition> points;
		private GeneralPath path;
		public PointList() {
			points = new ArrayList<ScreenPosition>();
			path = new GeneralPath();
			path = new GeneralPath();
		}

		public void add(ScreenPosition p) {
			add(p.getX(), p.getY());
		}

		public void add(int x, int y) {
			points.add(new ScreenPosition(x, y));
			if (points.size() > 1) {
				path.lineTo(x, y);
			} else {
				path.moveTo(x, y);
			}
		}

		public ScreenPosition get(int index) {
			return points.get(index);
		}

		public int size() {
			return points.size();
		}

		public void paint(Graphics2D g) {
			g.draw(path);
		}

		public Iterator<ScreenPosition> iterator() {
			return points.iterator();
		}
	}

}