/*
 * PROJECT: matereal at http://mr.digitalmuseum.jp/
 * ----------------------------------------------------------------------------
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is matereal.
 *
 * The Initial Developer of the Original Code is Jun KATO.
 * Portions created by the Initial Developer are
 * Copyright (C) 2009 Jun KATO. All Rights Reserved.
 *
 * Contributor(s): Jun KATO
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */
package sample.test.pen;

import java.awt.BorderLayout;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JButton;

import java.awt.FlowLayout;
import java.awt.CardLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JLabel;
import java.awt.Font;

public class WizardFrame extends JFrame {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JPanel jNavigationPanel = null;
	private JButton jNextButton = null;
	private JButton jBackButton = null;
	private JPanel jConfigurationPanel = null;
	private transient CardLayout cardLayout;
	private transient final JComponent[] components;
	private transient int index = 0;
	private JPanel jTitlePanel = null;
	private JLabel jTitleLabel = null;

	/**
	 * This is the default constructor
	 */
	public WizardFrame(JComponent[] components) {
		super();
		initialize();
		this.components = components;
		for (JComponent card : components) {
			getJConfigurationPanel().add(card, card.getName());
		}
		loadWizardComponent();
	}

	public void next() {
		unloadWizardComponent();
		cardLayout.next(getJConfigurationPanel());
		index = (index + 1) % components.length;
		loadWizardComponent();
	}

	public void previous() {
		unloadWizardComponent();
		cardLayout.previous(getJConfigurationPanel());
		index = (index + components.length - 1) % components.length;
		loadWizardComponent();
	}

	private void unloadWizardComponent() {
		if (components[index] instanceof WizardComponent) {
			((WizardComponent)components[index]).onUnload();
		}
	}

	private void loadWizardComponent() {
		getJTitleLabel().setText("Step."+(index + 1)+" "+components[index].getName());
		if (components[index] instanceof WizardComponent) {
			((WizardComponent)components[index]).onLoad();
		}
	}

	/**
	 * This method initializes this
	 *
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.setContentPane(getJContentPane());
		this.setTitle("JFrame");
	}

	/**
	 * This method initializes jContentPane
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJTitlePanel(), BorderLayout.NORTH);
			jContentPane.add(getJConfigurationPanel(), BorderLayout.CENTER);
			jContentPane.add(getJNavigationPanel(), BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jNavigationPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getJNavigationPanel() {
		if (jNavigationPanel == null) {
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setAlignment(FlowLayout.RIGHT);
			jNavigationPanel = new JPanel();
			jNavigationPanel.setLayout(flowLayout);
			jNavigationPanel.add(getJBackButton(), null);
			jNavigationPanel.add(getJNextButton(), null);
		}
		return jNavigationPanel;
	}

	/**
	 * This method initializes jNextButton
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getJNextButton() {
		if (jNextButton == null) {
			jNextButton = new JButton();
			jNextButton.setText("Next");
			jNextButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) { next(); }
			});
		}
		return jNextButton;
	}

	/**
	 * This method initializes jBackButton
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getJBackButton() {
		if (jBackButton == null) {
			jBackButton = new JButton();
			jBackButton.setText("Back");
			jBackButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) { previous(); }
			});
		}
		return jBackButton;
	}

	/**
	 * This method initializes jConfigurationPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getJConfigurationPanel() {
		if (jConfigurationPanel == null) {
			cardLayout = new CardLayout();
			cardLayout.setHgap(5);
			cardLayout.setVgap(5);
			jConfigurationPanel = new JPanel();
			jConfigurationPanel.setLayout(cardLayout);
		}
		return jConfigurationPanel;
	}

	/**
	 * This method initializes jTitlePanel
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getJTitlePanel() {
		if (jTitlePanel == null) {
			FlowLayout flowLayout1 = new FlowLayout();
			flowLayout1.setAlignment(FlowLayout.LEFT);
			jTitlePanel = new JPanel();
			jTitlePanel.setLayout(flowLayout1);
			jTitlePanel.add(getJTitleLabel(), null);
		}
		return jTitlePanel;
	}

	private JLabel getJTitleLabel() {
		if (jTitleLabel == null) {
			jTitleLabel = new JLabel();
			jTitleLabel.setText("JLabel");
			jTitleLabel.setFont(new Font("Dialog", Font.BOLD, 24));
		}
		return jTitleLabel;
	}

}
