/*
 * PROJECT: napkit at http://matereal.sourceforge.jp/
 * ----------------------------------------------------------------------------
 *
 * This file is part of NyARToolkit Application Toolkit.
 * NyARToolkit Application Toolkit, or simply "napkit",
 * is a simple wrapper library for NyARToolkit.
 *
 * ----------------------------------------------------------------------------
 *
 * License version: GPL 3.0
 *
 * napkit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * napkit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with napkit. If not, see <http://www.gnu.org/licenses/>.
 */
package jp.digitalmuseum.napkit;

import java.awt.image.BufferedImage;

import jp.nyatla.nyartoolkit.NyARException;
import jp.nyatla.nyartoolkit.core.raster.rgb.NyARRgbRaster_BasicClass;
import jp.nyatla.nyartoolkit.core.rasterreader.INyARBufferReader;
import jp.nyatla.nyartoolkit.core.rasterreader.INyARRgbPixelReader;
import jp.nyatla.nyartoolkit.core.types.NyARIntSize;

/**
 * The simplest INyARRgbRaster implementation for byte[] image data.
 *
 * @author Jun KATO
 */
public class NapByteRasterBGR extends NyARRgbRaster_BasicClass implements NapRaster
{
	private static int BUFFERFORMAT
			= INyARBufferReader.BUFFERFORMAT_BYTE1D_B8G8R8_24;

	/** Image object */
	private BufferedImage image;
	/** Image data */
	private byte[] pixels;

	/** Pixel reader */
	private INyARRgbPixelReader pixelReader = new INyARRgbPixelReader() {
		public void getPixel(int x, int y, int[] pixels_int) throws NyARException {
			final int offset = (x + y * _size.w) * 3;
			pixels_int[0] = (pixels[offset + 2] & 0xff); // R
			pixels_int[1] = (pixels[offset + 1] & 0xff); // G
			pixels_int[2] = (pixels[offset + 0] & 0xff); // B
			return;
		}
		public void getPixelSet(int[] x, int[] y, int num, int[] pixels_int) throws NyARException {
			int offsetDest = num * 3 - 1;
			for (int i = num - 1; i >= 0; i--) {
				final int offset = (x[i] + y[i] * _size.w) * 3;
				pixels_int[offsetDest --] = (pixels[offset + 0] & 0xff);// B
				pixels_int[offsetDest --] = (pixels[offset + 1] & 0xff);// G
				pixels_int[offsetDest --] = (pixels[offset + 2] & 0xff);// R
			}
			return;
		}
		public void setPixel(int iX, int iY, int[] iRgb) throws NyARException {
			final byte
				r = (byte)(iRgb[0] - 0x80),
				g = (byte)(iRgb[1] - 0x80),
				b = (byte)(iRgb[2] - 0x80);
			final int offset = (iX + iY*NapByteRasterBGR.this._size.w)*3;
			pixels[offset + 0] = r;
			pixels[offset + 1] = g;
			pixels[offset + 2] = b;
		}
		public void setPixels(int[] iX, int[] iY, int iNum, int[] iIntrgb)
				throws NyARException {
			NyARException.notImplement();
		}
	};

	/** Buffer reader */
	private INyARBufferReader bufferReader = new INyARBufferReader() {
		public Object getBuffer() { return pixels; }
		public int getBufferType() { return BUFFERFORMAT; }
		public boolean isEqualBufferType(int type) { return type == BUFFERFORMAT; }
	};

	/**
	 * Constructor with width and height in pixels specified.
	 */
	public NapByteRasterBGR(int width, int height) {
		super(new NyARIntSize(width, height));
	}

	/**
	 * Set pixel data.
	 * @param pixels pixel data in a byte array.
	 */
	public void setData(final byte[] pixels) { this.pixels = pixels.clone(); }

	@Override public INyARRgbPixelReader getRgbPixelReader() { return pixelReader; }
	public INyARBufferReader getBufferReader() { return bufferReader; }

	public BufferedImage getImage() {
		if (image == null ||
				image.getWidth() != _size.w ||
				image.getHeight() != _size.h) {
			image = new BufferedImage(_size.w, _size.h, BufferedImage.TYPE_3BYTE_BGR);
		}
		image.getRaster().setDataElements(0, 0, _size.w, _size.h, pixels);
		return image;
	}
}
